// ------------------------------------------------------
// AstroClock applet
// Version 3.19
// Copyright(c) 2006 Ali Adams
//
// http://www.geocities.com/aliadams
//
// Easy to read 24 hour analog clock with
// day/night views on current day for any location on earth
// and optional Islamic prayer times assumed as follows:
//
// Fajr    : BeginAstronomicalTwilight
// Dhuhr   : SunTransit
// Asr     : (EndCivilTwilight + SunTransit) / 2
// Mughrib : Sunset or EndCivilTwilight
// Ashaa   : EndAstronomicalTwilight
//
// AstroClock applet enhances:
// JavaClock applet
// Version 2.0
// Copyright(c) 1998, 1999 Antony Pranata
// http://www.antonypr.pair.com
//
// Java Clock
// used to display either analog and digital clock
//
// This source code is available freely. You may only use
// it for your personal purpose only(please do not use it
// for commercial purpose).
//
// You may also use this applet on your web page freely
// as long as credit is given(a link to my home page
// is enough).
//
// What's new in this version?
// - This new version of Java Clock support DST
//   I added this feature in this version because many users
//   asked me for it. Actually, I just knew DST when a user
//   ask me: Do your Java Clock support DST? FYI, I live in
//   Indonesia, South East Asia and we do not use DST here
//  (our country lie on equator line).
//
// ------------------------------------------------------
// Modified Feb 2000 to make a 24 hour analog clock
// Pete Boardman - pete.boardman@pobox.com
//
//
// ------------------------------------------------------
//
// ------------------------------------------------------
import java.awt.*;
import java.applet.*;
import java.util.*;
import java.net.*;
import java.text.*;

// ------------------------------------------------------
// AstroClock class
// ------------------------------------------------------
public class AstroClock extends Applet implements Runnable
{
  public static final Object[][] colors =
    { { "BLACK",     Color.black     },
      { "BLUE",      Color.blue      },
      { "CYAN",      Color.cyan      },
      { "DARKGRAY",  Color.darkGray  },
      { "GRAY",      Color.gray      },
      { "GREEN",     Color.green     },
      { "LIGHTGRAY", Color.lightGray },
      { "MAGENTA",   Color.magenta   },
      { "ORANGE",    Color.orange    },
      { "PINK",      Color.pink      },
      { "RED",       Color.red       },
      { "WHITE",     Color.white     },
      { "YELLOW",    Color.yellow    } };


  // The first field is the name of a city
  // The second field is a difference time from GMT
  // The third field is 1 if the city support DST, 0 otherwise
  // The fourth field is the latitude
  // The fifth field is the longitude
  // The sixth field is the elevation
  public static final Object[][] cityZone =
    {
		// REF: http://www.wunderground.com
		{ "Auckland",		"720",	"1",	"-37",	"174.8",	"6"},
		{ "Sydney",			"600",	"1",	"-34",	"151.2",	"3"},
		{ "Adelaide",		"570",	"1",	"-34.9","138.5",	"4"},
		{ "Tokyo",			"540",	"0",	"35.8",	"140",		"5"},
		{ "Kuala Lumpur",	"480",	"1",	"3.1",	"101.6",	"21"},
		{ "Jakarta",		"420",	"0",	"-6.2",	"106.8",	"0"},
		{ "New Delhi",		"330",	"0",	"28.6",	"77.1",		"233"},
		{ "Abu Dhabi",		"240",	"0",	"24.4",	"54.7",		"27"},
		{ "Tehran",			"210",	"1",	"35.7",	"51.3",		"1191"},
		{ "Baghdad",		"180",	"1",	"33.2",	"44.2",		"34"},
		{ "Beirut",			"120",	"1",	"33.8",	"35.5",		"29"},
		{ "Gdansk",			"60",	"1",	"54.4",	"18.7",		"138"},
		{ "London",			"0",	"1",	"51.4",	"-0.4",		"24"},
		{ "Buenos Aires",	"-180",	"1",	"-34.6","-58.4",	"6"},
		{ "Santiago",		"-240",	"1",	"-33.4","-70.8",	"475"},
		{ "New York",		"-300",	"1",	"40.8",	"-74",		"27"},
		{ "Mexico City",	"-360",	"1",	"19.4",	"-99.2",	"2303"},
		{ "Phoenix",		"-420",	"0",	"33.5",	"-112.1",	"346"},
		{ "Los Angeles",	"-480",	"1",	"34",	"-118.2",	"78"},
		{ "Nome, Alaska",	"-540",	"1",	"64.8",	"-165.2",	"13"},
		{ "Hilo, Hawaii",	"-600",	"0",	"19.7",	"-155.1",	"11"}
	};


  private Thread threadClock; // thread of the clock
  private boolean isAnalog = true; // true if analog clock is displayed
  private Font   fontText = null; // font used to display numbers on analog dial - not in 12h
  private String fontFace = "Arial"; // font used to display numbers on analog dial - not in 12h
  private Color  fontColor = Color.black; // font color;
  private int    fontSize = 10; // font size;
  private Font  digitalclockfontText = null; // font used to display digital clock
  private Color digitalclockfontColor = Color.red; // digitalclock font color;
  private int   digitalclockfontSize = 20; // digitalclock font size;
  private Color backColor = Color.white; // background color
  private Color hHandColor = Color.darkGray; // hour hand color
  private Color mHandColor = Color.blue; // minute hand color
  private Color sHandColor = Color.black; // second hand color
  private Color hourColor = Color.red; // hour point color
  private Color minuteColor = Color.lightGray; // minute point color

  private int xPoint[] = new int[4];
  private int yPoint[] = new int[4];

  private Image backImage; // background image
  private MediaTracker tracker;
  private Image imageBuffer; // buffer image

  private boolean showLocationName;
  private int fromGMT; // local time - GMT time
  private int minutesFromGMT; // current time zone
  private boolean isDST; // true if the city/country use DST
  private boolean isLocalUseDST; // true if local time use DST
  private int oldHour   = -1;
  private int oldMinute = -1;
  private int oldSecond = -1;

  //pb new stuff
  private boolean noonAtTop = true; // true if noon at top, 24h at bottom
  private boolean drawHourNumbers = true; // draw hour numbers on dial
  private boolean drawHourPoints = false; // draw hour points on dial
  private boolean drawMinuteNumbers = true; // draw minute numbers on dial
  private boolean drawMinutePoints = true; // draw minute points on dial

  private GeoLocation location;
  private String locationName;
  private double latitude;
  private double longitude;
  private double elevation;
  private boolean DST;
  private String timeZoneName;
  private TimeZone timeZone;
  private AstronomicalCalendar astro;
  private boolean showPrayerTimes;

  private String website;



  // ----------------------------------------------------
  // setCity method, set current city info
  // ----------------------------------------------------
  public void setCity(String city) {

	// set default values
	minutesFromGMT = fromGMT;
	isDST          = isLocalUseDST;

	boolean cityFound = false;
	for(int i = 0; i < cityZone.length; i++) {
	  if(city.indexOf((String)cityZone[i][0]) != -1)
	  {
		locationName   =                     (String)cityZone[i][0];
		minutesFromGMT =  Integer.parseInt  ((String)cityZone[i][1]);
		int hShift            = (int)minutesFromGMT / (int)60;
		int mShift            = (int)minutesFromGMT % (int)60;
		if ((hShift == 0) && (mShift == 0)) {
			timeZoneName   = "GMT";
		}
		else
		{
			if (hShift > 0) {
				timeZoneName = "GMT+" +
				 ((hShift < 10) ?("0" + hShift) : Integer.toString(hShift)) + ":" +
				 ((mShift < 10) ?("0" + mShift) : Integer.toString(mShift));
			}
			else {
				timeZoneName = "GMT-" +
				 ((Math.abs(hShift) < 10) ?("0" + Math.abs(hShift)) : Integer.toString(Math.abs(hShift))) + ":" +
				 ((Math.abs(mShift) < 10) ?("0" + Math.abs(mShift)) : Integer.toString(Math.abs(mShift)));
			}
		}
		/*
		String IDs[] = timeZone.getAvailableIDs();
		for (int i=0; i<IDs.length; i++)
		{
			System.out.println(IDs[i]);
		}
		*/
		isDST          = (Integer.parseInt  ((String)cityZone[i][2]) == 1);
		latitude       = (Double.parseDouble((String)cityZone[i][3]));
		longitude      = (Double.parseDouble((String)cityZone[i][4]));
		elevation      = (Double.parseDouble((String)cityZone[i][5]));
		cityFound      = true;
		break;
	  }
	}

	if (cityFound)
	{
		timeZone = TimeZone.getTimeZone(timeZoneName);
//		setIsLocalUseDST(isDST); // timeZone doesn't have DST the way timeZoneName is built
		location = new GeoLocation(locationName, latitude, longitude, elevation, timeZone);
		astro = new AstronomicalCalendar(location);
	}
	else
	{
		// load default location
		loadLocationFromHTMLFile();
	}
  }


  // ----------------------------------------------------
  // setIsLocalUseDST method
  // ----------------------------------------------------
  public void setIsLocalUseDST(boolean usedst) {
    isLocalUseDST = usedst;
  }

  // ----------------------------------------------------
  // findColor method, parse a string into color
  // ----------------------------------------------------
  private Color findColor(String param, Color defColor) {
    if(param != null) {
      param = param.toUpperCase();
      if(param.charAt(0) == '#') {
        return new Color(Integer.parseInt(param.substring(1), 16));
      }
      else {
        for(int i = 0; i < colors.length; i++)
          if(param.compareTo((String)colors[i][0]) == 0) {
            return(Color)colors[i][1];
          }
      }
    }
    return defColor;
  }


  // ----------------------------------------------------
  // loadLocationFromHTMLFile method, load default location
  // ----------------------------------------------------
  private void loadLocationFromHTMLFile()
  {
    String param;

    locationName = getParameter("location");
    if(locationName == null)
      locationName = "Abu Dhabi";

    timeZoneName = getParameter("timezone");
    if(timeZoneName == null)
      timeZoneName = "GMT+04:00";
	timeZone = TimeZone.getTimeZone(timeZoneName);

    // find out local time zone and use it
	Date localDate = new Date();
	fromGMT = -localDate.getTimezoneOffset();
	minutesFromGMT = fromGMT;


    // "dst" parameter
    param = getParameter("dst");
    if((param != null) &&(param.indexOf("false") > -1))
      isLocalUseDST = false;
    else
      isLocalUseDST = true;
    isDST = isLocalUseDST;


    // "showlocation" parameter
    param = getParameter("showlocation");
    if((param != null) &&(param.indexOf("false") > -1))
      showLocationName = false;
    else
      showLocationName = true;


    try {
      latitude = Double.parseDouble(getParameter("latitude"));
    }
    catch(NumberFormatException e) {
      latitude = 24.48;
    }

    try {
      longitude = Double.parseDouble(getParameter("longitude"));
    }
    catch(NumberFormatException e) {
      longitude = 54.45;
    }

    try {
      elevation = Double.parseDouble(getParameter("elevation"));
    }
    catch(NumberFormatException e) {
      elevation = 23;
    }

	location = new GeoLocation(locationName, latitude, longitude, elevation, timeZone);
	astro = new AstronomicalCalendar(location);
  }



  // ----------------------------------------------------
  // init method, applet initialization
  // ----------------------------------------------------
  public void init() {
    // Read "typeface" & "fontsize" parameter from HTML file
    String param = getParameter("typeface");
    if(param == null)
      param = "Helvetica";
	fontFace = param;

    try {
      fontSize = Integer.parseInt(getParameter("fontsize"));
    }
    catch(NumberFormatException e) {
      fontSize = 10;
    }
    fontText = new Font(param, Font.PLAIN, fontSize);
    fontColor = findColor(getParameter("fontcolor"), fontColor);

    try {
      digitalclockfontSize = Integer.parseInt(getParameter("digitalclockfontsize"));
    }
    catch(NumberFormatException e) {
      digitalclockfontSize = 14;
    }
    digitalclockfontText = new Font(param, Font.PLAIN, digitalclockfontSize);
    digitalclockfontColor = findColor(getParameter("digitalclockfontcolor"), digitalclockfontColor);

	// load default location
	loadLocationFromHTMLFile();

    // Read color parameter from HTML file
    backColor = findColor(getParameter("backcolor"), backColor);
    hHandColor = findColor(getParameter("hhandcolor"), hHandColor);
	hHandColor = new Color(hHandColor.getRed(), hHandColor.getGreen(), hHandColor.getBlue(), 128);
    mHandColor = findColor(getParameter("mhandcolor"), mHandColor);
	mHandColor = new Color(mHandColor.getRed(), mHandColor.getGreen(), mHandColor.getBlue(), 128);
    sHandColor = findColor(getParameter("shandcolor"), sHandColor);
    hourColor = findColor(getParameter("hourcolor"), hourColor);
    minuteColor = findColor(getParameter("minutecolor"), minuteColor);

    // Read "backimage" parameter from HTML file
    tracker    = new MediaTracker(this);
    param = getParameter("backimage");
    if(param != null) {
      backImage = getImage(getCodeBase(), param);
      tracker.addImage(backImage, 0);
    }
    else
      backImage = null;

    // "analog" parameter
    param = getParameter("analog");
    if((param != null) &&(param.indexOf("false") > -1))
      isAnalog = false;
    else
      isAnalog = true;



    // "noonattop" parameter
    param = getParameter("noonattop");
    if((param != null) &&(param.indexOf("false") > -1))
      noonAtTop = false;
    else
      noonAtTop = true;

    // "hournumbers" parameter
    param = getParameter("hournumbers");
    if((param != null) &&(param.indexOf("false") > -1))
      drawHourNumbers = false;
    else
      drawHourNumbers = true;

    // "hourpoints" parameter
    param = getParameter("hourpoints");
    if((param != null) &&(param.indexOf("false") > -1))
      drawHourPoints = false;
    else
      drawHourPoints = true;

    // "minutenumbers" parameter
    param = getParameter("minutenumbers");
    if((param != null) &&(param.indexOf("false") > -1))
      drawMinuteNumbers = false;
    else
      drawMinuteNumbers = true;

    // "minutepoints" parameter
    param = getParameter("minutepoints");
    if((param != null) &&(param.indexOf("false") > -1))
      drawMinutePoints = false;
    else
      drawMinutePoints = true;

    // "prayers" parameter
    param = getParameter("prayers");
    if((param != null) &&(param.indexOf("false") > -1))
	  showPrayerTimes = false;
    else
	  showPrayerTimes = true;


    website = getParameter("website");
    if(website == null)
      website = "http://www.geocities.com/aliadams";

  }

  // ----------------------------------------------------
  // start method
  // ----------------------------------------------------
  public void start() {
    // Create the buffer
    if(imageBuffer == null) {
      Dimension dim = size();
      imageBuffer = createImage(dim.width, dim.height);
    }

    // Create the thread
    if(threadClock == null) {
      threadClock = new Thread(this);
      threadClock.start();
    }
  }

  // ----------------------------------------------------
  // stop method
  // ----------------------------------------------------
  public void stop() {
    // Stop the thread
    if(threadClock != null) {
      threadClock.stop();
      threadClock = null;
      imageBuffer = null;
    }
  }

  // ----------------------------------------------------
  // run method
  // ----------------------------------------------------
  public void run() {
    // Wait until background image loaded
    try {
      tracker.waitForAll();
    }
    catch(InterruptedException e) {
      return;
    }

    // If current thread is threadClock then ...
    while(Thread.currentThread() == threadClock) {
      // Draw the clock
      repaint();

      // Delay for 500 ms
      try {
        threadClock.sleep(500);
      }
      catch(InterruptedException e) {
        break;
      }
    }
  }

  // ----------------------------------------------------
  // IsFirstAprLastOct method
  // ----------------------------------------------------
  private boolean IsFirstAprLastOct(Date checkDate) {
    int dd = checkDate.getDate();
    int mm = checkDate.getMonth();
    if((mm >= 3) &&(mm <= 9)) {
      Date dateApril = new Date(checkDate.getYear(), 3, 1);
      int firstWeekApril = 8 - dateApril.getDay();
      if(firstWeekApril > 7)
        firstWeekApril -= 7;

      Date dateOct = new Date(checkDate.getYear(), 9, 31);
      int lastWeekOct = 31 - dateOct.getDay();

      boolean isSetAhead1 =       ((mm == 3) &&(dd > firstWeekApril)) ||
       ((mm == 3) &&(dd == firstWeekApril) &&(checkDate.getHours() >= 2)) ||
       (mm > 3);
      boolean isSetAhead2 =
       ((mm == 9) &&(dd < lastWeekOct)) ||
       ((mm == 9) &&(dd == lastWeekOct) &&(checkDate.getHours() < 2)) ||
       (mm < 9);
      if(isSetAhead1 && isSetAhead2)
        return true;
      else
        return false;
    }
    return false;
  }

  // ----------------------------------------------------
  // update method
  // ----------------------------------------------------
  public void update(Graphics g) {

	// update clock time
    Date newTime = new Date();
    long timeDest = newTime.getTime();
    timeDest += (minutesFromGMT - fromGMT) * 60 * 1000;
    newTime = new Date(timeDest);

    int hour   = newTime.getHours();

    // Set DST
    if (latitude >= 0) // northen hemisphere
    {
		if((isDST) && (!isLocalUseDST)) {
		  if(IsFirstAprLastOct(newTime))
			hour += 1;
		}

		if((!isDST) && (isLocalUseDST)) {
		  Date newTime2 = new Date();
		  if(IsFirstAprLastOct(newTime2))
			hour -= 1;
		}
	}
	else // if (latitude < 0) { // southern hemisphere
	{
		if((isDST) && (!isLocalUseDST)) {
		  if(!IsFirstAprLastOct(newTime))
			hour += 1;
		}

		if((!isDST) && (isLocalUseDST)) {
		  Date newTime2 = new Date();
		  if(!IsFirstAprLastOct(newTime2))
			hour -= 1;
		}
	}

    int minute = newTime.getMinutes();
    int second = newTime.getSeconds();


	// re-calc astro times for both clock and prayer times
	// should calc timezone here from minutesFromGMT
	timeZone = TimeZone.getTimeZone(timeZoneName);
	location = new GeoLocation(locationName, latitude, longitude, elevation, timeZone);
	astro = new AstronomicalCalendar(location);



    //if((hour != oldHour) ||(minute != oldMinute) || (second != oldSecond))
    {

      // Draw background in the buffer
      Graphics graphBuffer = imageBuffer.getGraphics();
      DrawBackground(graphBuffer);


      if(isAnalog) {

		Dimension dim = size();
		int CenterX = dim.width >> 1;
		int CenterY = dim.height >> 1;

        // Prepare the drawing of the analog clock
        double radius1 = Math.min(CenterX, CenterY) * 0.75;
        double radius2 = Math.min(CenterX, CenterY) * 0.04;

        // Draw minute hand
        double deltaMinuteTheta = 0;
        if((minute > 15) &&(minute < 45))
		  deltaMinuteTheta = +(Math.PI / 60.0) * 0.3; // shift hand forward a fraction of a minute
        else
		  deltaMinuteTheta = -(Math.PI / 60.0) * 0.5; // shift hand forward a fraction of a minute

		//double posMinute = Math.PI *(minute / 30.0 + second / 1800.0) + deltaMinuteTheta;
		double posMinute = Math.PI *(minute / 30.0 + 0 / 1800.0) + deltaMinuteTheta;
        xPoint[0] =(int)Math.round(CenterX - 2 * radius2 * Math.sin(posMinute)) - 1;
        xPoint[1] =(int)Math.round(CenterX - radius2 * Math.cos(posMinute));
        xPoint[2] =(int)Math.round(CenterX + radius1 * Math.sin(posMinute)) + 1;
        xPoint[3] =(int)Math.round(CenterX + radius2 * Math.cos(posMinute));

        yPoint[0] =(int)Math.round(CenterY + 2 * radius2 * Math.cos(posMinute)) - 1;
        yPoint[1] =(int)Math.round(CenterY - radius2 * Math.sin(posMinute));
        yPoint[2] =(int)Math.round(CenterY - radius1 * Math.cos(posMinute)) + 1;
        yPoint[3] =(int)Math.round(CenterY + radius2 * Math.sin(posMinute));
        graphBuffer.setColor(mHandColor);
        graphBuffer.fillPolygon(xPoint, yPoint, 4);

        if(minute < 30)
          graphBuffer.setColor(Color.lightGray);
        else
          graphBuffer.setColor(Color.darkGray);
        graphBuffer.drawLine(xPoint[0], yPoint[0], xPoint[1], yPoint[1]);
        graphBuffer.drawLine(xPoint[1], yPoint[1], xPoint[2], yPoint[2]);

        if(minute < 30)
          graphBuffer.setColor(Color.darkGray);
        else
          graphBuffer.setColor(Color.lightGray);
        graphBuffer.drawLine(xPoint[2], yPoint[2], xPoint[3], yPoint[3]);
        graphBuffer.drawLine(xPoint[3], yPoint[3], xPoint[0], yPoint[0]);


        // Draw hour hand
        // for 12h clock was: double posHour = Math.PI *(hour / 6.0 + minute / 360.0);
        double noonOffset;
        if(noonAtTop)
        	noonOffset = Math.PI;
        else
        	noonOffset = 0;

        double posHour = noonOffset +(Math.PI *(hour / 12.0 + minute / 720.0));
        double sinposHour = Math.sin(posHour);
        double cosposHour = Math.cos(posHour);

        radius1 = Math.min(CenterX, CenterY) * 0.5;
        radius2 = Math.min(CenterX, CenterY) * 0.05;
        xPoint[0] =(int)Math.round(CenterX - 2 * radius2 * sinposHour) - 1;
        xPoint[1] =(int)Math.round(CenterX - radius2 * cosposHour);
        xPoint[2] =(int)Math.round(CenterX + radius1 * sinposHour) + 1;
        xPoint[3] =(int)Math.round(CenterX + radius2 * cosposHour);

        yPoint[0] =(int)Math.round(CenterY + 2 * radius2 * cosposHour) - 1;
        yPoint[1] =(int)Math.round(CenterY - radius2 * sinposHour);
        yPoint[2] =(int)Math.round(CenterY - radius1 * cosposHour) + 1;
        yPoint[3] =(int)Math.round(CenterY + radius2 * sinposHour);

        graphBuffer.setColor(hHandColor);
        graphBuffer.fillPolygon(xPoint, yPoint, 4);

        if((hour >= 12) &&(hour <= 24)) // was more complicated for 12h
          graphBuffer.setColor(Color.lightGray);
        else
          graphBuffer.setColor(Color.darkGray);
        graphBuffer.drawLine(xPoint[0], yPoint[0], xPoint[1], yPoint[1]);
        graphBuffer.drawLine(xPoint[1], yPoint[1], xPoint[2], yPoint[2]);

        if((hour >= 0) &&(hour <= 12)) // was more complicated for 12h
          graphBuffer.setColor(Color.darkGray);
        else
          graphBuffer.setColor(Color.lightGray);
        graphBuffer.drawLine(xPoint[2], yPoint[2], xPoint[3], yPoint[3]);
        graphBuffer.drawLine(xPoint[3], yPoint[3], xPoint[0], yPoint[0]);


        // Draw second hand
        radius1 = Math.min(CenterX, CenterY) * 0.75;
        graphBuffer.setColor(sHandColor);
        double posSecond = Math.PI * second / 30.0;
        graphBuffer.drawLine(
          CenterX, CenterY,
         (int)Math.round(CenterX + radius1 * Math.sin(posSecond)),
         (int)Math.round(CenterY - radius1 * Math.cos(posSecond)));

        // Draw digital time
        graphBuffer.setFont(digitalclockfontText);
        graphBuffer.setColor(digitalclockfontColor);

        String displayTime =
         ((hour < 10) ?("0" + hour) : Integer.toString(hour)) + ":" +
         ((minute < 10) ?("0" + minute) : Integer.toString(minute)) + ":" +
         ((second < 10) ?("0" + second) : Integer.toString(second));

        FontMetrics fm = graphBuffer.getFontMetrics();
        int LokY =(int)(dim.height * 0.3);
        if(LokY < 0)
          LokY = 0;

        int LokX =(dim.width - fm.stringWidth(displayTime)) >> 1;
        if(LokX < 0)
          LokX = 0;

        graphBuffer.drawString(displayTime, LokX, LokY);


        // Draw digital date
        graphBuffer.setFont(fontText);
        graphBuffer.setColor(fontColor);
		SimpleDateFormat Dateformatter = new SimpleDateFormat ("yyyy-MM-dd");
		Date today = new Date();
		String todayString = Dateformatter.format(today);
		int length = todayString.length();
		graphBuffer.drawString(todayString, CenterX - (int)((length*0.5)*(fontSize*0.5)), LokY + 12);

      }
      else {
        // draw location info and paryer times
        // with digital clock and date
      }

      oldHour = hour;
      oldMinute = minute;
      oldSecond = second;
    }

    // Copy the buffer to the "screen"
    g.drawImage(imageBuffer, 0, 0, null);
  }



  // ----------------------------------------------------
  // paint method
  // ----------------------------------------------------
  public void paint(Graphics g) {
    // If there's error, draw nothing to the screen
    Dimension dim = size();
    if(tracker.isErrorAny()) {
      g.setColor(Color.white);
      g.fillRect(0, 0, dim.width, dim.height);
      return;
    }

    // If there's NO error, draw the image to the screen
    if(tracker.checkAll(true))
      DrawBackground(g);
  }


  // ----------------------------------------------------
  // DrawBackground method
  // ----------------------------------------------------
  private void DrawBackground(Graphics g) {
    // Fill background with solid color
    Dimension dim = size();
    g.setColor(backColor);
    g.fillRect(0, 0, dim.width, dim.height);

    // Draw background image and stretch to fill window
    if(backImage != null) {
	  g.drawImage(backImage, 0, 0, dim.width, dim.height, null);
    }



	int CenterX = dim.width  >> 1;
	int CenterY = dim.height >> 1;

    if(isAnalog) {

      // Draw the border.
      g.setColor(new Color(128, 128, 128, 64));
      g.drawOval(0, 0, dim.width, dim.height);
      g.fillOval(0, 0, dim.width, dim.height);


	  // switched around for anti-clockwise drawing direction
	  int sunriseAngle =(int)( 270 -(( astro.getSunrise().getHours() +(astro.getSunrise().getMinutes()/60.0) ) *(360.0 / 24.0)) );
	  int sunsetAngle  =(int)( -90 -(( astro.getSunset().getHours() +(astro.getSunset().getMinutes()/60.0) ) *(360.0 / 24.0)) );
	  int fajrAngle    =(int)( 270 -(( astro.getBeginAstronomicalTwilight().getHours() +(astro.getBeginAstronomicalTwilight().getMinutes()/60.0) ) *(360.0 / 24.0)) );
	  int transitAngle =(int)( 180 +((sunriseAngle + sunsetAngle) / 2) );
	  int mughribAngle =(int)( -90 -(( astro.getEndCivilTwilight().getHours() +(astro.getEndCivilTwilight().getMinutes()/60.0) ) *(360.0 / 24.0)) );
	  int asrAngle     =(int)( 180 +((mughribAngle + transitAngle) / 2) );
	  int ashaaAngle   =(int)( -90 -(( astro.getEndAstronomicalTwilight().getHours() +(astro.getEndAstronomicalTwilight().getMinutes()/60.0) ) *(360.0 / 24.0)) );
      int rad     = (int)(Math.min(dim.width, dim.height) * 0.92);


	  int noonOffset;
	  if(noonAtTop)
		noonOffset = 0;
	  else
		noonOffset = 180;

		// Draw night.
		//g.setColor(new Color(128, 0, 255, 64));
		//g.fillArc(0, 0, dim.width, dim.height, noonOffset + sunsetAngle, (sunsetAngle + sunriseAngle));

		// Draw day.
		g.setColor(new Color(255, 255, 255, 128));
		g.fillArc(0, 0, dim.width, dim.height, noonOffset + sunriseAngle, 360 - (sunriseAngle - sunsetAngle));

		if(showPrayerTimes)
		{
		  // Draw Fajr.
		  g.setColor(new Color(255, 255, 255, 128));
		  g.fillArc(0, 0, dim.width, dim.height, noonOffset + fajrAngle-1, 2);

		  // Draw Dhuhr.
		  g.setColor(new Color(255, 255, 0, 128));
		  g.fillArc(0, 0, dim.width, dim.height, noonOffset + transitAngle-2, 2);

		  // Draw Asr.
		  g.setColor(new Color(208, 255, 255, 128));
		  g.fillArc(0, 0, dim.width, dim.height, noonOffset + asrAngle-4, 2);

		  // Draw Mughrib.
		  g.setColor(new Color(255, 192, 192, 128));
		  g.fillArc(0, 0, dim.width, dim.height, noonOffset + mughribAngle, 2);

		  // Draw Ashaa. // 15min after AstroTwEnd --> add to angle 15degree = 1hr, so quarter hour == 4 degrees
		  g.setColor(new Color(0, 0, 0, 128));
		  g.fillArc(0, 0, dim.width, dim.height, noonOffset + ashaaAngle-4, 2);
		}


		// Draw the moon circle.
		/*
		int moon_r = 50;
		g.setColor(bgcolor);
		g.drawOval(CenterX - moon_r/2, CenterY - moon_r/2, moon_r, moon_r);
		g.fillOval(CenterX - moon_r/2, CenterY - moon_r/2, moon_r, moon_r);
		*/




		if(drawHourPoints) {
		  // Draw hour marks
		  double radius = Math.min(CenterX, CenterY) * 0.92;
		  for(int i = 0; i <= 24; i++) {  // was 12
			double buffer = Math.PI *(0.5 - i / 12.0); // was 6.0
			int posX =(int)Math.floor(CenterX + radius * Math.cos(buffer));
			int posY =(int)Math.floor(CenterY - radius * Math.sin(buffer));

			g.setColor(hourColor);
			g.fill3DRect(posX - 2, posY - 2, 4, 4, true);
		  } // draw hours
		}


		if(drawMinutePoints) {
		  // Draw minute
	      int tickLen = 0;
	      double radius = Math.min(CenterX, CenterY) * 0.8; // new radius for minutes
	      for(int i = 0; i <= 60; i++) {
	          double buffer = Math.PI * i / 30.0;
	          int posX =(int)Math.floor(CenterX + radius * Math.cos(buffer));
	          int posY =(int)Math.floor(CenterY - radius * Math.sin(buffer));

			  if((i % 5) == 0) // draw bigger minutePoints at 5 intervals
			  {
				if(drawMinuteNumbers)
				  tickLen = 0; // don't draw point, number will come here
				else
				  tickLen = 4;
			  }
			  else
			  {
				tickLen = 2;
			  }
			  g.setColor(minuteColor);
			  g.fill3DRect(posX - 1, posY - 1, tickLen, tickLen, false);
		  }
		}

		Font oldFont = fontText;
		fontText = new Font(fontFace, Font.BOLD, fontSize);
		if(drawHourNumbers) {
		  // draw numbers
		  // new - not in 12h original
		  g.setColor(hourColor);
		  g.setFont(fontText);
		  String hourNumber;

		  double radius = Math.min(CenterX, CenterY) * 0.92;

		  int segments = 24;
		  double theta = 0;
		  if(noonAtTop)
			theta =(2 * Math.PI) -(Math.PI / 2);
		  else
			theta =(2 * Math.PI) +(Math.PI / 2);
		  double delta =(2 * Math.PI) / segments;
		  for(int i=0; i<segments; i++)
		  {
			int dx = 0;
			if(i>=10)	// two digits
				dx	= -(fontSize/4) -(fontSize/2) + 1; // manual fix
			else 		// one digit
				dx	= -(fontSize/4);					// manual fix
			int dy	= +(fontSize/3) + 1;				// manual fix

			int x	=(int)Math.floor(CenterX +(radius * Math.cos(theta)) + dx);
			int y	=(int)Math.floor(CenterY -(radius * Math.sin(theta)) + dy);
			theta	= theta - delta;
			hourNumber = i + "";
			g.drawString(hourNumber, x, y);
		  }
		}
		fontText = oldFont;


		if(drawMinuteNumbers) {
		  // draw numbers
		  // new - not in 12h original
		  g.setColor(minuteColor);
		  g.setFont(fontText);
		  String minuteNumber;

		  double radius = Math.min(CenterX, CenterY) * 0.8;

		  int segments = 12;
		  double theta =(Math.PI / 2);
		  double delta =(2 * Math.PI) / segments;
		  for(int i=0; i<segments; i++)
		  {
			int dx = 0;
			if(i>=2)	// two digits
				dx	= -(fontSize/4) -(fontSize/2) + 1; // manual fix
			else 		// one digit
				dx	= -(fontSize/4);					// manual fix
			int dy	= +(fontSize/3) + 1;				// manual fix

			int x	=(int)Math.floor(CenterX +(radius * Math.cos(theta)) + dx);
			int y	=(int)Math.floor(CenterY -(radius * Math.sin(theta)) + dy);
			theta	= theta - delta;
			minuteNumber = i*5 + "";
			g.drawString(minuteNumber, x, y);
		  }
	    }

		// Draw location name at bottom half of analog clock
		if(showLocationName) {
			if (locationName != null) {
				int length = locationName.length();
				double radius = Math.min(CenterX, CenterY) * 0.8;
				g.drawString(locationName, CenterX - (int)((length*0.5)*(fontSize*0.5)), CenterX + (int)radius - 20);
			}
		}


    } //isAnalog
    else
    {
		String locationInfo1 = locationName;
		String locationInfo2 = latitude + " " + longitude + ", " + elevation + "m";
		String locationInfo3 = timeZone.getID();
		String locationInfo4 = " " + (DST ? "DST" : "");
		SimpleDateFormat Dateformatter = new SimpleDateFormat ("dd/MM/yyyy");
		Date today = new Date();
		String todayString = Dateformatter.format(today);

		g.setColor(fontColor);
		g.setFont(fontText);
		g.drawString(locationInfo1, 10, 15);
		g.drawString(todayString  , 120, 15);
		g.drawString(locationInfo2, 10, 30);
		g.drawString(locationInfo3, 10, 45);
		g.drawString(locationInfo4, 70, 45);

		if(showPrayerTimes)
		{
		  // Format the current time.
		  SimpleDateFormat Timeformatter = new SimpleDateFormat ("HH:mm:ss");
		  Date   prayerTime;
		  String prayerString;

		  prayerTime = astro.getBeginAstronomicalTwilight();
		  prayerString = Timeformatter.format(prayerTime);
		  String Fajr	 = "Fajr------ :     " + prayerString;

		  prayerTime = astro.getSunrise();
		  prayerString = Timeformatter.format(prayerTime);
		  String Sunrise = "Sunrise--- :   " + prayerString;

		  prayerTime = astro.getSunTransit();
		  prayerString = Timeformatter.format(prayerTime);
		  String Dhuhr	 = "Dhuhr----- :   " + prayerString;

		  prayerTime = new Date(((astro.getEndCivilTwilight().getTime() + astro.getSunTransit().getTime()) / 2));
		  prayerString = Timeformatter.format(prayerTime);
		  String Asr	 = "Asr------- :    " + prayerString;

		  prayerTime = astro.getSunset();
		  prayerString = Timeformatter.format(prayerTime);
		  String Sunset	 = "Sunset---- :   " + prayerString;

		  prayerTime = astro.getEndCivilTwilight();
		  prayerString = Timeformatter.format(prayerTime);
		  String Mughrib = "Mughrib--- :   " + prayerString;

		  prayerTime = astro.getEndAstronomicalTwilight();
		  prayerString = Timeformatter.format(prayerTime);
		  String Ashaa	 = "Ashaa----- :  " + prayerString;

		  g.drawString("    PRAYER TIMES    "   , 10, 75);
		  g.drawString(Fajr   , 10,  85);
		  g.drawString(Sunrise, 10, 100);
		  g.drawString(Dhuhr  , 10, 115);
		  g.drawString(Asr    , 10, 130);
		  g.drawString(Sunset , 10, 145);
		  g.drawString(Mughrib, 10, 160);
		  g.drawString(Ashaa  , 10, 175);
		  g.drawString(website  , 15, 195);
		}
	}
  }


  // ----------------------------------------------------
  // mouseDown method
  // ----------------------------------------------------
  public boolean mouseDown(Event evt, int x, int y)
  {
	if ( (!isAnalog) && (y > 180) && (y < 210) ) {
      if (website != null)
      {
         try
         {
            URL url = new URL(getDocumentBase(), website);
            getAppletContext().showDocument(url, "_self");
			stop(); // before going out to website
         }
         catch(MalformedURLException e)
         {
            e.printStackTrace();
         }
      }
	}
	else {
	  isAnalog = !isAnalog;
	}
    return(true);
  }

  // ----------------------------------------------------
  // mouseEnter method
  // ----------------------------------------------------
  public boolean mouseEnter(Event evt, int x, int y)
  {
      if (website != null)
      {
		showStatus(website);
      }
      return(true);
  }

  // ----------------------------------------------------
  // mouseExit method
  // ----------------------------------------------------
  public boolean mouseExit(Event evt, int x, int y)
  {
      if (website != null)
      {
		showStatus("");
      }
      return(true);
  }

}
