/**
 * A class that represents a numeric time. Times that represent a time of day
 * are stored as {@link java.util.Date}s in this API. The time class is used to
 * represent numeric time such as the time in hours, minutes, seconds and
 * milliseconds of a
 * {@link net.sourceforge.Time.AstronomicalCalendar#getTemporalHour() temporal hour}.
 *
 * @author &copy; Eliyahu Hershfeld 2004 - 2005
 * @version 0.9.0
 */
public class Time {
	private static final int SECOND_MILLIS = 1000;

	private static final int MINUTE_MILLIS = SECOND_MILLIS * 60;

	private static final int HOUR_MILLIS = MINUTE_MILLIS * 60;

	private int hours = 0;

	private int minutes = 0;

	private int seconds = 0;

	private int milliseconds = 0;

	public Time(int hours, int minutes, int seconds, int milliseconds) {
		this.hours = hours;
		this.minutes = minutes;
		this.seconds = seconds;
		this.milliseconds = milliseconds;
	}

	public Time(double millis) {
		this((int) millis);
	}

	public Time(int millis) {
		hours = (int) (millis / HOUR_MILLIS);
		millis = millis - hours * HOUR_MILLIS;

		minutes = (int) (millis / MINUTE_MILLIS);
		millis = millis - minutes * MINUTE_MILLIS;

		seconds = (int) (millis / SECOND_MILLIS);
		millis = millis - seconds * SECOND_MILLIS;

		milliseconds = (int) millis;
	}

	/**
	 * @return Returns the hour.
	 */
	public int getHours() {
		return hours;
	}

	/**
	 * @param hours
	 *            The hours to set.
	 */
	public void setHours(int hours) {
		this.hours = hours;
	}

	/**
	 * @return Returns the minutes.
	 */
	public int getMinutes() {
		return minutes;
	}

	/**
	 * @param minutes
	 *            The minutes to set.
	 */
	public void setMinutes(int minutes) {
		this.minutes = minutes;
	}

	/**
	 * @return Returns the seconds.
	 */
	public int getSeconds() {
		return seconds;
	}

	/**
	 * @param seconds
	 *            The seconds to set.
	 */
	public void setSeconds(int seconds) {
		this.seconds = seconds;
	}

	/**
	 * @return Returns the milliseconds.
	 */
	public int getMilliseconds() {
		return milliseconds;
	}

	/**
	 * @param milliseconds
	 *            The milliseconds to set.
	 */
	public void setMilliseconds(int milliseconds) {
		this.milliseconds = milliseconds;
	}

	public double getTime() {
		return hours * HOUR_MILLIS + minutes * MINUTE_MILLIS + seconds
				* SECOND_MILLIS + milliseconds;
	}

	public String toString() {
		return new TimeFormatter().format(this);
	}
}
